@@ -37,6 +37,13 @@
 namespace cupt {
 namespace internal {
 
+enum class MetadataWorker::IndexType { Packages, PackagesDiff, Localization, LocalizationFile, LocalizationFileDiff };
+
+bool MetadataWorker::__is_diff_type(const IndexType& indexType)
+{
+	return indexType == IndexType::PackagesDiff || indexType == IndexType::LocalizationFileDiff;
+}
+
 string MetadataWorker::__get_indexes_directory() const
 {
 	return _config->getPath("cupt::directory::state::lists");
@@ -86,7 +93,14 @@ std::function< string () > generateMovingSub(const string& downloadPath, const s
 {
 	return [downloadPath, targetPath]() -> string
 	{
-		return fs::move(downloadPath, targetPath);
+		if (fs::move(downloadPath, targetPath))
+		{
+			return "";
+		}
+		else
+		{
+			return format2e(__("unable to rename '%s' to '%s'"), downloadPath, targetPath);
+		}
 	};
 };
 
@@ -117,12 +131,12 @@ bool generateUncompressingSub(const download::Uri& uri, const string& downloadPa
 		}
 		else
 		{
-			fatal2("internal error: extension '%s' has no uncompressor", filenameExtension);
+			fatal2i("extension '%s' has no uncompressor", filenameExtension);
 		}
 
 		if (::system(format2("which %s >/dev/null", uncompressorName).c_str()))
 		{
-			warn2("'%s' uncompressor is not available, not downloading '%s'",
+			warn2(__("the '%s' uncompressor is not available, not downloading '%s'"),
 					uncompressorName, string(uri));
 			return false;
 		}
@@ -135,7 +149,7 @@ bool generateUncompressingSub(const download::Uri& uri, const string& downloadPa
 			unlink(downloadPath.c_str());
 			if (uncompressingResult)
 			{
-				return format2(__("failed to uncompress '%s', '%s' returned error %d"),
+				return format2(__("failed to uncompress '%s', '%s' returned the error %d"),
 						downloadPath, uncompressorName, uncompressingResult);
 			}
 			return string(); // success
@@ -150,7 +164,7 @@ bool generateUncompressingSub(const download::Uri& uri, const string& downloadPa
 	}
 	else
 	{
-		warn2("unknown file extension '%s', not downloading '%s'",
+		warn2(__("unknown file extension '%s', not downloading '%s'"),
 					filenameExtension, string(uri));
 		return false;
 	}
@@ -160,6 +174,17 @@ string __get_pidded_string(const string& input)
 {
 	return format2("(%d): %s", getpid(), input);
 }
+template < typename... Args >
+string piddedFormat2(const string& format, const Args&... args)
+{
+	return __get_pidded_string(format2(format, args...));
+}
+template < typename... Args >
+string piddedFormat2e(const string& format, const Args&... args)
+{
+	return __get_pidded_string(format2e(format, args...));
+}
+
 string __get_download_log_message(const string& longAlias)
 {
 	return __get_pidded_string(string("downloading: ") + longAlias);
@@ -217,7 +242,7 @@ bool MetadataWorker::__update_release(download::Manager& downloadManager,
 
 				try
 				{
-					cachefiles::getReleaseInfo(*_config, targetPath);
+					cachefiles::getReleaseInfo(*_config, targetPath, targetPath);
 				}
 				catch (Exception& e)
 				{
@@ -261,16 +286,14 @@ bool MetadataWorker::__update_release(download::Manager& downloadManager,
 				return moveError;
 			}
 
-			if (!cachefiles::verifySignature(*_config, targetPath))
+			if (!cachefiles::verifySignature(*_config, targetPath, longAlias))
 			{
-				warn2("signature verification for '%s' failed", longAlias);
-
 				if (!_config->getBool("cupt::update::keep-bad-signatures"))
 				{
 					// for compatibility with APT tools delete the downloaded file
 					if (unlink(signatureTargetPath.c_str()) == -1)
 					{
-						warn2e("unable to delete file '%s'", signatureTargetPath);
+						warn2e(__("unable to remove the file '%s'"), signatureTargetPath);
 					}
 				}
 			}
@@ -351,7 +374,7 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 	auto fail = [baseLongAlias, &cleanUp]()
 	{
 		cleanUp();
-		warn2("%s: failed to proceed", baseLongAlias);
+		warn2(__("%s: failed to proceed"), baseLongAlias);
 	};
 
 	try
@@ -361,8 +384,8 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 			File diffIndexFile(diffIndexPath, "r", openError);
 			if (!openError.empty())
 			{
-				logger->loggedFatal(Logger::Subsystem::Metadata, 3,
-						__get_pidded_string(format2e("unable to open the file '%s'", diffIndexPath)));
+				logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
+						piddedFormat2e, "unable to open the file '%s': %s", diffIndexPath, openError);
 			}
 
 			TagParser diffIndexParser(&diffIndexFile);
@@ -383,8 +406,8 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 						const string& line = *lineIt;
 						if (!regex_match(line, m, checksumsLineRegex))
 						{
-							logger->loggedFatal(Logger::Subsystem::Metadata, 3,
-									__get_pidded_string(format2("malformed 'hash-size-name' line '%s'", line)));
+							logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
+									piddedFormat2, "malformed 'hash-size-name' line '%s'", line);
 						}
 
 						if (isHistory)
@@ -402,21 +425,21 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 					auto values = internal::split(' ', string(fieldValue));
 					if (values.size() != 2)
 					{
-						logger->loggedFatal(Logger::Subsystem::Metadata, 3,
-								__get_pidded_string(format2("malformed 'hash-size' line '%s'", string(fieldValue))));
+						logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
+								piddedFormat2, "malformed 'hash-size' line '%s'", string(fieldValue));
 					}
 					wantedHashSum = values[0];
 				}
 			}
 			if (wantedHashSum.empty())
 			{
-				logger->loggedFatal(Logger::Subsystem::Metadata, 3,
-						__get_pidded_string("failed to find wanted hash sum"));
+				logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
+						piddedFormat2, "failed to find the target hash sum");
 			}
 		}
 		if (unlink(diffIndexPath.c_str()) == -1)
 		{
-			warn2("unable to delete a temporary index file '%s'", diffIndexPath);
+			warn2(__("unable to remove the file '%s'"), diffIndexPath);
 		}
 
 		HashSums subTargetHashSums;
@@ -427,8 +450,8 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 
 		if (::system(format2("cp %s %s", targetPath, patchedPath).c_str()))
 		{
-			logger->loggedFatal(Logger::Subsystem::Metadata, 3,
-					__get_pidded_string(format2("unable to copy '%s' to '%s'", targetPath, patchedPath)));
+			logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
+					piddedFormat2, "unable to copy '%s' to '%s'", targetPath, patchedPath);
 		}
 
 		while (currentSha1Sum != wantedHashSum)
@@ -445,8 +468,8 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 				}
 				else
 				{
-					logger->loggedFatal(Logger::Subsystem::Metadata, 3,
-							__get_pidded_string(format2("unable to find a patch for the sha1 sum '%s'", currentSha1Sum)));
+					logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
+							piddedFormat2, "unable to find a patch for the sha1 sum '%s'", currentSha1Sum);
 				}
 			}
 
@@ -454,8 +477,8 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 			auto patchIt = patches.find(patchName);
 			if (patchIt == patches.end())
 			{
-				logger->loggedFatal(Logger::Subsystem::Metadata, 3,
-						__get_pidded_string(format2("unable to a patch entry for the patch '%s'", patchName)));
+				logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
+						piddedFormat2, "unable to find a patch entry for the patch '%s'", patchName);
 			}
 
 			string patchSuffix = "/" + patchName + ".gz";
@@ -508,7 +531,7 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 			 out:
 				if (unlink(unpackedPath.c_str()) == -1)
 				{
-					warn2e("unable to remove partial index patch file '%s'", unpackedPath);
+					warn2e(__("unable to remove the file '%s'"), unpackedPath);
 				}
 				return result;
 			};
@@ -521,10 +544,10 @@ bool __download_and_apply_patches(download::Manager& downloadManager,
 			}
 		}
 
-		auto moveError = fs::move(patchedPath, targetPath);
-		if (!moveError.empty())
+		if (!fs::move(patchedPath, targetPath))
 		{
-			logger->loggedFatal(Logger::Subsystem::Metadata, 3, __get_pidded_string(moveError));
+			logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
+					piddedFormat2e, "unable to rename '%s' to '%s'", patchedPath, targetPath);
 		}
 		return true;
 	}
@@ -541,27 +564,24 @@ bool MetadataWorker::__download_index(download::Manager& downloadManager,
 		const string& targetPath, bool sourceFileChanged)
 {
 	bool simulating = _config->getBool("cupt::worker::simulate");
-	if (indexType == IndexType::PackagesDiff && !fs::fileExists(targetPath))
+	if (__is_diff_type(indexType) && !fs::fileExists(targetPath))
 	{
 		return false; // nothing to patch
 	}
 
 	const string& uri = downloadRecord.uri;
 	auto downloadPath = baseDownloadPath;
-	switch (indexType)
+	if (__is_diff_type(indexType))
 	{
-		case IndexType::Packages:
-		case IndexType::Localization:
-		case IndexType::LocalizationFile:
-			downloadPath += getFilenameExtension(uri);
-			break;
-		case IndexType::PackagesDiff:
-			downloadPath += ".diffindex";
-			break;
-	};
+		downloadPath += ".diffindex";
+	}
+	else
+	{
+		downloadPath += getFilenameExtension(uri);
+	}
 
 	std::function< string () > uncompressingSub;
-	if (indexType == IndexType::PackagesDiff || indexType == IndexType::Localization)
+	if (__is_diff_type(indexType) || indexType == IndexType::Localization)
 	{
 		uncompressingSub = []() -> string { return ""; }; // is not a final file
 	}
@@ -582,7 +602,7 @@ bool MetadataWorker::__download_index(download::Manager& downloadManager,
 		{
 			if (unlink(downloadPath.c_str()) == -1)
 			{
-				warn2e("unable to remove an outdated partial file '%s'", downloadPath);
+				warn2e(__("unable to remove an outdated partial file '%s'"), downloadPath);
 			}
 		}
 	}
@@ -602,7 +622,7 @@ bool MetadataWorker::__download_index(download::Manager& downloadManager,
 		{
 			if (unlink(downloadPath.c_str()) == -1)
 			{
-				warn2e("unable to remove partial index file '%s'", downloadPath);
+				warn2e(__("unable to remove the file '%s'"), downloadPath);
 			}
 			return __("hash sums mismatch");
 		}
@@ -612,7 +632,7 @@ bool MetadataWorker::__download_index(download::Manager& downloadManager,
 			vector< download::Manager::DownloadEntity >{ downloadEntity });
 	if (indexType != IndexType::Packages && !simulating && downloadError.empty())
 	{
-		if (indexType == IndexType::PackagesDiff)
+		if (__is_diff_type(indexType))
 		{
 			return __download_and_apply_patches(downloadManager, downloadRecord,
 					indexEntry, baseDownloadPath, downloadPath, targetPath, _logger);
@@ -630,32 +650,49 @@ bool MetadataWorker::__download_index(download::Manager& downloadManager,
 	}
 }
 
-bool MetadataWorker::__update_index(download::Manager& downloadManager,
-		const cachefiles::IndexEntry& indexEntry, bool releaseFileChanged, bool& indexFileChanged)
+
+struct MetadataWorker::IndexUpdateInfo
+{
+	IndexType type;
+	string targetPath;
+	vector< cachefiles::FileDownloadRecord > downloadInfo;
+	string label;
+};
+
+bool MetadataWorker::__update_main_index(download::Manager& downloadManager,
+		const cachefiles::IndexEntry& indexEntry, bool releaseFileChanged, bool& mainIndexFileChanged)
 {
 	// downloading Packages/Sources
-	auto targetPath = cachefiles::getPathOfIndexList(*_config, indexEntry);
+	IndexUpdateInfo info;
+	info.type = IndexType::Packages;
+	info.targetPath = cachefiles::getPathOfIndexList(*_config, indexEntry);
+	info.downloadInfo = cachefiles::getDownloadInfoOfIndexList(*_config, indexEntry);
+	info.label = __("index");
+	return __update_index(downloadManager, indexEntry,
+			std::move(info), releaseFileChanged, mainIndexFileChanged);
+}
 
-	auto downloadInfo = cachefiles::getDownloadInfoOfIndexList(*_config, indexEntry);
-
-	indexFileChanged = true;
+bool MetadataWorker::__update_index(download::Manager& downloadManager, const cachefiles::IndexEntry& indexEntry,
+		IndexUpdateInfo&& info, bool sourceFileChanged, bool& thisFileChanged)
+{
+	thisFileChanged = true;
 
 	// checking maybe there is no difference between the local and the remote?
 	bool simulating = _config->getBool("cupt::worker::simulate");
-	if (!simulating && fs::fileExists(targetPath))
+	if (!simulating && fs::fileExists(info.targetPath))
 	{
-		FORIT(downloadRecordIt, downloadInfo)
+		FORIT(downloadRecordIt, info.downloadInfo)
 		{
-			if (downloadRecordIt->hashSums.verify(targetPath))
+			if (downloadRecordIt->hashSums.verify(info.targetPath))
 			{
 				// yeah, really
-				indexFileChanged = false;
+				thisFileChanged = false;
 				return true;
 			}
 		}
 	}
 
-	auto baseDownloadPath = getDownloadPath(targetPath);
+	auto baseDownloadPath = getDownloadPath(info.targetPath);
 
 	{ // sort download files by priority and size
 		auto comparator = [this](const cachefiles::FileDownloadRecord& left, const cachefiles::FileDownloadRecord& right)
@@ -671,7 +708,7 @@ bool MetadataWorker::__update_index(download::Manager& downloadManager,
 				return (leftPriority > rightPriority);
 			}
 		};
-		std::sort(downloadInfo.begin(), downloadInfo.end(), comparator);
+		std::sort(info.downloadInfo.begin(), info.downloadInfo.end(), comparator);
 	}
 
 	bool useIndexDiffs = _config->getBool("cupt::update::use-index-diffs");
@@ -684,27 +721,42 @@ bool MetadataWorker::__update_index(download::Manager& downloadManager,
 
 	const string diffIndexSuffix = ".diff/Index";
 	auto diffIndexSuffixSize = diffIndexSuffix.size();
-	FORIT(downloadRecordIt, downloadInfo)
+	FORIT(downloadRecordIt, info.downloadInfo)
 	{
 		const string& uri = downloadRecordIt->uri;
 		bool isDiff = (uri.size() >= diffIndexSuffixSize &&
 				!uri.compare(uri.size() - diffIndexSuffixSize, diffIndexSuffixSize, diffIndexSuffix));
-		if (isDiff && !useIndexDiffs)
+		auto indexType = info.type;
+		if (isDiff)
 		{
-			continue;
+			if (!useIndexDiffs)
+			{
+				continue;
+			}
+			if (indexType == IndexType::Packages)
+			{
+				indexType = IndexType::PackagesDiff;
+			}
+			else if (indexType == IndexType::LocalizationFile)
+			{
+				indexType = IndexType::LocalizationFileDiff;
+			}
+			else
+			{
+				continue; // unknown diff type
+			}
 		}
-		auto indexType = isDiff ? IndexType::PackagesDiff : IndexType::Packages;
 
 		if(__download_index(downloadManager, *downloadRecordIt, indexType, indexEntry,
-				baseDownloadPath, targetPath, releaseFileChanged))
+				baseDownloadPath, info.targetPath, sourceFileChanged))
 		{
 			return true;
 		}
 	}
 
 	// we reached here if neither download URI succeeded
-	warn2("failed to download index for '%s/%s'",
-			indexEntry.distribution, indexEntry.component);
+	warn2(__("failed to download %s for '%s/%s'"),
+			info.label, indexEntry.distribution, indexEntry.component);
 	return false;
 }
 
@@ -725,7 +777,7 @@ bool MetadataWorker::__download_translations(download::Manager& downloadManager,
 		auto fail = [localizationIndexLongAlias, &cleanUp]()
 		{
 			cleanUp();
-			warn2("failed to parse localization info from '%s'", localizationIndexLongAlias);
+			warn2(__("failed to parse localization data from '%s'"), localizationIndexLongAlias);
 		};
 
 		try
@@ -734,8 +786,8 @@ bool MetadataWorker::__download_translations(download::Manager& downloadManager,
 			File localizationIndexFile(localizationIndexPath, "r", openError);
 			if (!openError.empty())
 			{
-				logger->loggedFatal(Logger::Subsystem::Metadata, 3,
-						__get_pidded_string(format2e("unable to open the file '%s'", localizationIndexPath)));
+				logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
+						piddedFormat2e, "unable to open the file '%s': %s", localizationIndexPath, openError);
 			}
 
 			TagParser localizationIndexParser(&localizationIndexFile);
@@ -753,8 +805,8 @@ bool MetadataWorker::__download_translations(download::Manager& downloadManager,
 						const string& line = *lineIt;
 						if (!regex_match(line, m, checksumsLineRegex))
 						{
-							logger->loggedFatal(Logger::Subsystem::Metadata, 3,
-									__get_pidded_string(format2("malformed 'hash-size-name' line '%s'", line)));
+							logger->loggedFatal2(Logger::Subsystem::Metadata, 3,
+									piddedFormat2, "malformed 'hash-size-name' line '%s'", line);
 						}
 
 						cachefiles::FileDownloadRecord record;
@@ -805,6 +857,22 @@ bool MetadataWorker::__download_translations(download::Manager& downloadManager,
 void MetadataWorker::__update_translations(download::Manager& downloadManager,
 		const cachefiles::IndexEntry& indexEntry, bool indexFileChanged)
 {
+	auto downloadInfoV3 = cachefiles::getDownloadInfoOfLocalizedDescriptions3(*_config, indexEntry);
+	if (!downloadInfoV3.empty()) // full info is available directly in Release file
+	{
+		for (const auto& record: downloadInfoV3)
+		{
+			IndexUpdateInfo info;
+			info.type = IndexType::LocalizationFile;
+			info.label = format2(__("'%s' descriptions localization"), record.language);
+			info.targetPath = record.localPath;
+			info.downloadInfo = record.fileDownloadRecords;
+			bool unused;
+			__update_index(downloadManager, indexEntry, std::move(info), indexFileChanged, unused);
+		}
+		return;
+	}
+
 	if (cachefiles::getDownloadInfoOfLocalizedDescriptions2(*_config, indexEntry).empty())
 	{
 		return;
@@ -851,7 +919,7 @@ bool MetadataWorker::__update_release_and_index_data(download::Manager& download
 
 	// phase 2
 	bool indexFileChanged;
-	if (!__update_index(downloadManager, indexEntry, releaseFileChanged, indexFileChanged))
+	if (!__update_main_index(downloadManager, indexEntry, releaseFileChanged, indexFileChanged))
 	{
 		return false;
 	}
@@ -884,7 +952,7 @@ void MetadataWorker::__list_cleanup(const string& lockPath)
 				cachefiles::getPathsOfLocalizedDescriptions(*_config, *indexEntryIt);
 		FORIT(pathIt, translationsPossiblePaths)
 		{
-			addUsedPrefix(*pathIt);
+			addUsedPrefix(pathIt->second);
 		}
 	}
 	addUsedPrefix(lockPath);
@@ -905,7 +973,7 @@ void MetadataWorker::__list_cleanup(const string& lockPath)
 			{
 				if (unlink(fileIt->c_str()) == -1)
 				{
-					warn2e("unable to delete '%s'", *fileIt);
+					warn2e(__("unable to remove the file '%s'"), *fileIt);
 				}
 			}
 		}
@@ -929,16 +997,13 @@ void MetadataWorker::updateReleaseAndIndexData(const shared_ptr< download::Progr
 			{
 				if (mkdir(indexesDirectory.c_str(), 0755) == -1)
 				{
-					_logger->loggedFatal(Logger::Subsystem::Metadata, 2,
-							format2e("unable to create the lists directory '%s'", indexesDirectory));
+					_logger->loggedFatal2(Logger::Subsystem::Metadata, 2,
+							format2e, "unable to create the lists directory '%s'", indexesDirectory);
 				}
 			}
 		}
 
-		if (!simulating)
-		{
-			lock.reset(new internal::Lock(*_config, lockFilePath));
-		}
+		lock.reset(new internal::Lock(*_config, lockFilePath));
 
 		{ // run pre-actions
 			_logger->log(Logger::Subsystem::Metadata, 2, "running apt pre-invoke hooks");
@@ -965,15 +1030,15 @@ void MetadataWorker::updateReleaseAndIndexData(const shared_ptr< download::Progr
 			{
 				if (mkdir(partialIndexesDirectory.c_str(), 0755) == -1)
 				{
-					_logger->loggedFatal(Logger::Subsystem::Metadata, 2,
-							format2e("unable to create partial directory '%s'", partialIndexesDirectory));
+					_logger->loggedFatal2(Logger::Subsystem::Metadata, 2,
+							format2e, "unable to create the directory '%s'", partialIndexesDirectory);
 				}
 			}
 		}
 	}
 	catch (...)
 	{
-		_logger->loggedFatal(Logger::Subsystem::Metadata, 1, "aborted downloading release and index data");
+		_logger->loggedFatal2(Logger::Subsystem::Metadata, 1, format2, "aborted downloading release and index data");
 	}
 
 	int masterExitCode = true;
@@ -988,7 +1053,7 @@ void MetadataWorker::updateReleaseAndIndexData(const shared_ptr< download::Progr
 			auto pid = fork();
 			if (pid == -1)
 			{
-				_logger->loggedFatal(Logger::Subsystem::Metadata, 2, format2e("fork failed"));
+				_logger->loggedFatal2(Logger::Subsystem::Metadata, 2, format2e, "%s() failed", "fork");
 			}
 
 			if (pid)
@@ -1019,7 +1084,7 @@ void MetadataWorker::updateReleaseAndIndexData(const shared_ptr< download::Progr
 			pid_t pid = wait(&status);
 			if (pid == -1)
 			{
-				_logger->loggedFatal(Logger::Subsystem::Metadata, 2, format2e("wait failed"));
+				_logger->loggedFatal2(Logger::Subsystem::Metadata, 2, format2e, "%s() failed", "wait");
 			}
 			pids.erase(pid);
 			// if something went bad in child, the parent won't return non-zero code too
@@ -1059,8 +1124,8 @@ void MetadataWorker::updateReleaseAndIndexData(const shared_ptr< download::Progr
 
 	if (!masterExitCode)
 	{
-		_logger->loggedFatal(Logger::Subsystem::Metadata, 1,
-				"there were errors while downloading release and index data");
+		_logger->loggedFatal2(Logger::Subsystem::Metadata, 1,
+				format2, "there were errors while downloading release and index data");
 	}
 }
 
